import { Client, Databases, Query, } from "node-appwrite";
import { Error, adapterGetUpdateData } from "lucia-sveltekit";
export const transformUserDoc = (doc) => {
    const docCopy = structuredClone(doc);
    const appWriteKeys = [
        "$read",
        "$write",
        "$id",
        "$createdAt",
        "$updatedAt",
        "$collection",
    ];
    const id = docCopy.$id;
    for (const keys in docCopy) {
        if (appWriteKeys.includes(keys)) {
            delete docCopy[keys];
        }
    }
    return {
        id,
        ...docCopy,
    };
};
const adapter = (config) => {
    const client = new Client();
    client
        .setEndpoint(config.api.endpoint)
        .setProject(config.project_id)
        .setKey(config.api.key);
    const db = new Databases(client, config.database.id);
    const userCollectionId = config.database.user_collection_id;
    const refreshTokenCollectionId = config.database.refresh_token_collection_id;
    return {
        getUserByRefreshToken: async (refreshToken) => {
            try {
                const refreshTokenDocs = await db.listDocuments(refreshTokenCollectionId, [Query.equal("refresh_token", refreshToken)], 1);
                const refreshTokenDoc = refreshTokenDocs.documents[0];
                if (!refreshTokenDoc)
                    return null;
                const userDoc = await db.getDocument(userCollectionId, refreshTokenDoc.user_id);
                return transformUserDoc(userDoc);
            }
            catch (e) {
                const error = e;
                if (error.code === 404)
                    return null; // document not found
                console.error(error);
                throw new Error("DATABASE_FETCH_FAILED");
            }
        },
        getUserByIdentifierToken: async (identifierToken) => {
            try {
                const userDocs = await db.listDocuments(userCollectionId, [Query.equal("identifier_token", identifierToken)], 1);
                return userDocs.documents[0]
                    ? transformUserDoc(userDocs.documents[0])
                    : null;
            }
            catch (e) {
                console.error(e);
                throw new Error("DATABASE_FETCH_FAILED");
            }
        },
        getUserById: async (userId) => {
            try {
                const userDoc = await db.getDocument(userCollectionId, userId);
                return transformUserDoc(userDoc);
            }
            catch (e) {
                const error = e;
                if (error.type === "document_not_found")
                    return null; // document not found
                console.error(e);
                throw new Error("DATABASE_FETCH_FAILED");
            }
        },
        setUser: async (userId, data) => {
            try {
                await db.createDocument(userCollectionId, userId, {
                    identifier_token: data.identifier_token,
                    hashed_password: data.hashed_password,
                    ...data.user_data,
                });
            }
            catch (e) {
                const error = e;
                if (error.type === "document_already_exists")
                    throw new Error("AUTH_DUPLICATE_USER_DATA");
                console.error(e);
                throw new Error("DATABASE_UPDATE_FAILED");
            }
        },
        deleteUser: async (userId) => {
            try {
                await db.deleteDocument(userCollectionId, userId);
            }
            catch (e) {
                console.error(e);
                throw new Error("DATABASE_UPDATE_FAILED");
            }
        },
        setRefreshToken: async (refreshToken, userId) => {
            try {
                await db.createDocument(refreshTokenCollectionId, "unique()", {
                    refresh_token: refreshToken,
                    user_id: userId,
                });
            }
            catch (e) {
                console.error(e);
                throw new Error("DATABASE_UPDATE_FAILED");
            }
        },
        deleteRefreshToken: async (refreshToken) => {
            try {
                const refreshTokenDocs = await db.listDocuments(refreshTokenCollectionId, [Query.equal("refresh_token", refreshToken)]);
                const promises = [];
                refreshTokenDocs.documents.forEach((val) => {
                    promises.push(db.deleteDocument(refreshTokenCollectionId, val.$id));
                });
                await Promise.all(promises);
            }
            catch (e) {
                console.error(e);
                throw new Error("DATABASE_UPDATE_FAILED");
            }
        },
        deleteUserRefreshTokens: async (userId) => {
            try {
                const refreshTokenDocs = await db.listDocuments(refreshTokenCollectionId, [Query.equal("user_id", userId)]);
                const promises = [];
                refreshTokenDocs.documents.forEach((val) => {
                    promises.push(db.deleteDocument(refreshTokenCollectionId, val.$id));
                });
                await Promise.all(promises);
            }
            catch (e) {
                console.error(e);
                throw new Error("DATABASE_UPDATE_FAILED");
            }
        },
        updateUser: async (userId, newData) => {
            const dbData = adapterGetUpdateData(newData);
            try {
                const userDoc = await db.updateDocument(userCollectionId, userId, dbData);
                return transformUserDoc(userDoc);
            }
            catch (e) {
                const error = e;
                if (error.type === "document_not_found")
                    throw new Error("AUTH_INVALID_USER_ID");
                console.error(error);
                throw new Error("DATABASE_UPDATE_FAILED");
            }
        },
    };
};
export default adapter;
